<?php
namespace Teknisa\FIN\Modules\ContaPagar\Controllers;

use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response\Error;
use Teknisa\Libs\Service\Login as LoginService;
use Symfony\Component\DependencyInjection\Loader\XmlFileLoader;
use Symfony\Component\DependencyInjection\ContainerBuilder;
use Symfony\Component\Config\FileLocator;
use Zeedhi\Framework\DependencyInjection\InstanceManager;
use Teknisa\FIN\Modules\ContaPagar\Services\AutenticacaoDDAService;

class AutenticacaoDDAController {

    protected $user;
    protected $password;
    protected $loginService;

    public function __construct(LoginService $loginService, AutenticacaoDDAService $autenticacaoDDAService) {
        $this->loginService           = $loginService;
        $this->autenticacaoDDAService = $autenticacaoDDAService;
        $this->getParameters();
    }

    private function getParameters() {
        try {
            $instanceManager = InstanceManager::getInstance();
            $container = $instanceManager->getContainer();
            $this->validateParameter($container);
            $this->dnsFinanceiro = $container->getParameter('dnsFinanceiro');
            $this->dnsProduto = $container->getParameter('dnsProduto');
            $this->projectId = $container->getParameter('projectid');
        } catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), 1));
        }
    }

    public function authenticate($request, $response) {
        $conditions = $request->getFilterCriteria()->getConditions();
        $bHash = $this->loginService->getBHash();
        try {
            $cookieRequestToken = $this->login();
            $token = $this->requestToken($cookieRequestToken, $bHash);

            $cookie = $this->startSession($token);

            $response->addDataSet(new DataSet('COOKIE', [$cookie]));
        } catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), 1));
        }
    }

    public function login(){
        //try {
            $loginUrl = $this->getUrlLogin();
            $ip = $this->loginService->getCurrentIp();
            $bHash = $this->loginService->getBHash();
            //$configurasmtp = $this->autenticacaoDDAService->getConfigurasmtp();
            //$login = $configurasmtp->getLoginsmtp();
            //$senha = $configurasmtp->getSenhasmtp();
            $login = 'suporte.odhen@teknisa.com';
            $senha = '1';

            /*$operador = $this->autenticacaoDDAService->getOperador();
            $senha = $operador->getDsemailoper();
            $login = $operador->getCdsenhaoperweb();*/


            $data = $this->buildData($ip, $bHash, $login, $senha, true);
            $header = $this->buildHeader();
            $result = $this->getServiceRemote($loginUrl.'/lm_login', $header, $data);
            preg_match_all('/^({"error":.*)$/mi', $result, $dataset);
            $this->validaErroRequisicao($result, $dataset);
            preg_match_all('/^Set-Cookie:\s*PHPSESSID=([^;]*)/mi', $result, $cookie);
            return $cookie[1][0];
        //} catch(\Exception $e) {
        //    throw $e;
        //}
    }

    public function requestToken($cookieLogin, $bHash) {
        $loginUrl = $this->getUrlLogin();
        $data = '{"CLIENT_ID":"http://libraries.teknisa.com/", "BHASH":"'.$bHash.'"}';
        $header = $this->buildHeader($data);
        $header[] = 'COOKIE: PHPSESSID='.$cookieLogin;
        $sessionLogin = curl_init($loginUrl.'/lm_requestToken');
        curl_setopt($sessionLogin, CURLOPT_HTTPHEADER, $header);
        curl_setopt($sessionLogin, CURLOPT_POST, true);
        curl_setopt($sessionLogin, CURLOPT_COOKIESESSION, true);
        curl_setopt($sessionLogin, CURLOPT_POSTFIELDS, $data);
        curl_setopt($sessionLogin, CURLOPT_RETURNTRANSFER, 1);
        $result = curl_exec($sessionLogin);
        $result = (array) (json_decode($result));
        curl_close($sessionLogin);
        return $result['dataset']->token[0];
    }

    public function startSession($tokenLogin) {
        $data = '{"filter":[{"name":"TOKEN","value":"'.$tokenLogin.'","operator":"="},{"name":"VERSION_FILE","value":null,"operator":"="}],"page":1,"requestType":"FilterData"}';
        $header = $this->buildHeader($data);
        $result = $this->getServiceRemote($this->dnsFinanceiro.'/lib_startSession', $header, $data);
        preg_match_all('/^({"error":.*)$/mi', $result, $dataset);
        $this->validaErroRequisicao($result, $dataset);
        preg_match_all('/^Set-Cookie:\s*PHPSESSID=([^;]*)/mi', $result, $cookie);
        return $cookie[1][0];
    }

    private function getUrlLogin() {
        $loginProperties = json_decode(file_get_contents(__DIR__.'/../../../../../../../../fin/backend/vendor/teknisa/libraries/config/config.json'), true);
        return $loginUrl = $loginProperties['loginUrl'];
    }

    private function buildData($ip, $bHash, $email, $password, $service) {
        return '{"filter":[{"name":"EMAIL","operator":"=","value":"'.$email.'"},{"name":"PASSWORD","operator":"=","value":"'.$password.'"},{"name":"REQUESTER_URL","operator":"=","value":"'.$this->dnsProduto.'"},{"name":"PRODUCT_ID","operator":"=","value":"'.$this->projectId.'"},{"name":"HOST","operator":"=","value":"false"},{"name":"PORT","operator":"=","value":"false"},{"name":"SERVICE","operator":"=","value":"'.$service.'"},{"name":"IP","operator":"=","value":"'.$ip.'"},{"name":"BHASH","operator":"=","value":"'.$bHash.'"}],"page":1,"requestType":"FilterData","origin":{"containerName":"LOGIN","widgetName":"LOGIN"}}';
    }

    private function buildHeader() {
        $header = array();
        $header[] = 'Content-Type: application/json';
        $header[] = 'Accept: application/json, text/plain, */*';
        $header[] = 'Connection: keep-alive';
        return $header;
    }

    public function getServiceRemote($url, $header, $data){
        $sessionLogin = curl_init($url);
        curl_setopt($sessionLogin, CURLOPT_HTTPHEADER, $header);
        curl_setopt($sessionLogin, CURLOPT_POST, true);
        curl_setopt($sessionLogin, CURLOPT_POSTFIELDS, $data);
        curl_setopt($sessionLogin, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($sessionLogin, CURLOPT_HEADER, 1);
        $result = curl_exec($sessionLogin);
        curl_close($sessionLogin);
        return $result;
    }

    public function validaErroRequisicao($result, $dataset){
        if ($dataset[0]) {
            $dataset = (array) (json_decode($dataset[1][0]));
            throw new \Exception($dataset['error']);
        } else {
            preg_match_all('/^({"dataset":.*)$/mi', $result, $dataset);
            $dataset = (array) (json_decode($dataset[1][0]));
            if (isset($dataset[0])) {
                throw new \Exception('Error while trying to login.');
            }
        }
    }

    private function validateParameter($container){
        if (!$container->hasParameter('projectId')) {
            throw new \Exception("Has no projectId parameter.");
        }
        if (!$container->hasParameter('dnsFinanceiro')) {
            throw new \Exception("Has no dnsFinanceiro parameter.");
        }
        if (!$container->hasParameter('dnsProduto')) {
            throw new \Exception("Has no dnsProduto parameter.");
        }
    }
}

?>