<?php

namespace Teknisa\FIN\Modules\ContaReceber\Controllers;

use Zeedhi\Framework\DTO;
use Teknisa\Libs\Service\CRUD as CrudService;
use Teknisa\Libs\Util\Environment;
use Zeedhi\Framework\DataSource\DataSet;
use Teknisa\FIN\Controllers\TeknisaCrud;
use Zeedhi\Framework\DataSource\ParameterBag;
use Teknisa\FIN\Modules\ContaReceber\Services\RateioContaService as RateioContaService;

/**
 * Class RateioContaController
 * @package Teknisa\FIN\Modules\ContaReceber\Controller
 */
class RateioContaController extends TeknisaCrud{

    protected $dataSourceName = 'ratctreceber_filial_centcust';

    /**
     * @param CrudService $crudService
     * @param ParameterBag $parameterBag
     * @param Environment $environment
     * @param RateioContaService $rateioContaService
     */
    public function __construct(
        CrudService $crudService, ParameterBag $parameterBag, Environment $environment, RateioContaService $rateioContaService
    ){
        $this->crudService  = $crudService;
        $this->parameterBag = $parameterBag;
        $this->environment  = $environment;
        $this->rateioContaService = $rateioContaService;
    }

    /**
     * Finaliza Inserção Rateio(s) Conta a Receber
     * @param DTO\Request\DataSet $dataset
     * @param DTO\Response $response
     */
    public function finalizaRateioConta(DTO\Request\DataSet $dataset, DTO\Response $response)
    {
        try {
            // Busca Dados do Dataset
            $dataset  = $dataset->getDataSet()->getRows();

            // Variáveis de Inicialização
            $nrorg        = $dataset[0]['nrorg'];
            $cdfilial     = $dataset[0]['cdfilial'];
            $nrlanctonf   = $dataset[0]['nrlanctonf'];
            $nrseqvenrec  = $dataset[0]['nrseqvenrec'];
            $nrparcelarec = $dataset[0]['nrparcelarec'];
            $rateios      = $dataset[1];

            // Finaliza Inserção Rateio(s) Conta a Receber
            $this->rateioContaService->finalizaRateioConta(
                $nrorg, $cdfilial, $nrlanctonf, $nrseqvenrec, $nrparcelarec, $rateios
            );

            // Retorna Resposta do Backend
            $response->addDataSet(new DataSet('FinalizaRateioConta', ['success' => 'S']));
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    /**
     * Deleta Rateio Conta a Receber
     * @param DTO\Request\Row $row
     * @param DTO\Response $response
     */
    public function deletaRateioConta(DTO\Request\Row $row, DTO\Response $response)
    {
        try {
            // Dados do Registro
            $row = $row->getRow();

            // Variáveis de Inicialização
            $nrorg        = $row['NRORG'];
            $cdfilial     = $row['CDFILIAL'];
            $nrlanctonf   = $row['NRLANCTONF'];
            $nrseqvenrec  = $row['NRSEQVENREC'];
            $nrparcelarec = $row['NRPARCELAREC'];
            $cdcentcust   = $row['CDCENTCUST'];

            // Deleta Rateio Conta a Receber
            $this->rateioContaService->deletaRateioConta(
                $nrorg, $cdfilial, $nrlanctonf, $nrseqvenrec, $nrparcelarec, $cdcentcust
            );

            // Registra Dados
            $response->addDataSet(new DataSet('DeletaRateioConta', ['success' => 'S']));
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

}