<?php

namespace Teknisa\FIN\Modules\ContaReceber\Services;

use Doctrine\ORM\EntityManager;
use Financeiro\Entities\Tprecboleto;
use Teknisa\Libs\Util\DateUtil;

/**
 * Class TipoRecebimentoService
 * @package Teknisa\FIN\Modules\ContaReceber\Services
 */
class TipoRecebimentoService {

    protected $entityManager;

    /**
     * @param EntityManager $entityManager
     */
    public function __construct(EntityManager $entityManager)
    {
        $this->entityManager = $entityManager;
    }

    /**
     * Finaliza Tipo de Recebimento Títulos a Receber/Recebidos
     * @param $nrorg
     * @param $cdfilial
     * @param $nrlanbolrec
     * @param $tiporecebimento
     * @throws \Exception
     */
    public function finalizaTipoRecebimento(
        $nrorg, $cdfilial, $nrlanbolrec, $tiporecebimento
    ){
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {

            // Percorre Tipo de Recebimento
            foreach ($tiporecebimento as $tprecebimento){

                // Busca Tipo de Recebimento
                /** @var Tprecboleto $tiporece */
                $tprecboleto = Tprecboleto::findOneByNrorgAndCdfilialAndNrlanbolrecAndCdtiporece(
                    $nrorg, $cdfilial, $nrlanbolrec, $tprecebimento['CDTIPORECE']
                );

                // Verifica não existencia Tipo de Recebimento
                if (!$tprecboleto) {
                    $tprecboleto = New Tprecboleto();
                    $tprecboleto->setNrorg($nrorg);
                    $tprecboleto->setCdfilial($cdfilial);
                    $tprecboleto->setNrlanbolrec($nrlanbolrec);
                }

                $tprecboleto->setNrseqtprecbo($tprecebimento['NRSEQTPRECBO']);
                $tprecboleto->setCdtiporece($tprecebimento['CDTIPORECE']);
                $tprecboleto->setCdcartao($tprecebimento['CDCARTAO']);
                $tprecboleto->setVrtprecbolr($tprecebimento['VRTPRECBOLR']);
                $tprecboleto->setNrcheqbolrec($tprecebimento['NRCHEQBOLREC']);
                $tprecboleto->setDtcheqbolrec(self::formataData($tprecebimento['DTCHEQBOLREC']));
                $tprecboleto->setCdbancbolrec($tprecebimento['CDBANCBOLREC']);
                $tprecboleto->setCdagenbolrec($tprecebimento['CDAGENBOLREC']);
                $tprecboleto->setCdcontbolrec($tprecebimento['CDCONTBOLREC']);
                $tprecboleto->setNrcarcrebolr($tprecebimento['NRCARCREBOLR']);
                $tprecboleto->setDscheqbolrec($tprecebimento['DSCHEQBOLREC']);
                $tprecboleto->setDtinclusao(DateUtil::getDataAtual());

                $this->entityManager->persist($tprecboleto);
            }

            $this->entityManager->flush();
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    /**
     * Deleta Tipo de Recebimento
     * @param $nrorg
     * @param $cdfilial
     * @param $nrlanbolrec
     * @param $tiporecebimento
     * @throws \Exception
     */
    public function deletaTipoRecebimento(
        $nrorg, $cdfilial, $nrlanbolrec, $tiporecebimento
    ){
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {

            // Busca Tipo de Recebimento
            /** @var Tprecboleto $tprecboletoatual */
            $tprecboletoatual = Tprecboleto::findOneByNrorgAndCdfilialAndNrlanbolrecAndCdtiporece(
                $nrorg, $cdfilial, $nrlanbolrec, $tiporecebimento[count($tiporecebimento)-1]['CDTIPORECE']
            );

            if((count($tiporecebimento)-2) > -1) {


                // Busca Tipo de Recebimento
                /** @var Tprecboleto $tprecboletoant */
                $tprecboletoant = Tprecboleto::findOneByNrorgAndCdfilialAndNrlanbolrecAndCdtiporece(
                    $nrorg, $cdfilial, $nrlanbolrec, $tiporecebimento[count($tiporecebimento) - 2]['CDTIPORECE']
                );

                if ($tprecboletoant) {
                    $tprecboletoant->setVrtprecbolr($tprecboletoant->getVrtprecbolr() + $tprecboletoatual->getVrtprecbolr());
                    $this->entityManager->persist($tprecboletoant);
                }
                $tprecboletoant->getVrtprecbolr();
                $tprecboletoant->persist();
            }

            $tprecboletoatual->delete();

            $this->entityManager->flush();
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    /**
     * Formata Data
     * @param $data
     * @return \DateTime|\Setforms2\Shared\ORM\DateTime|string
     */
    public static function formataData($data)
    {
        if (!$data)
            return null;

        if (strlen($data) == 19)
            $data = DateUtil::truncateString($data);
        $data = DateUtil::getDataDeString($data);
        $data = DateUtil::truncateData($data);

        return $data;
    }
}